package com.cultural.heritage.controller.book;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.bookingDate.single.BookingDateSingleAddRequest;
import com.cultural.heritage.model.dto.bookingTime.BookingTimeAddRequest;
import com.cultural.heritage.model.dto.bookingTime.single.BookingTimeSingleAddRequest;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.service.book.BookingDateService;
import com.cultural.heritage.service.book.BookingTimeService;
import com.cultural.heritage.service.common.CommonService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/book")
@Slf4j
@Tag(name = "写真预约日期管理模块")
public class BookingDateController {


    @Resource
    private BookingDateService bookingDateService;


    @Resource
    private BookingTimeService bookingTimeService;


    @Resource
    private CommonService commonService;



    /**
     * Web端管理员根据id删除预约日期
     * @param commonRequest 预约日期id
     * @return 是否删除成功
     */
    @PostMapping("/del/id")
    @Operation(summary = "Web端管理员根据id删除预约日期", description = "参数：预约日期id，权限：管理员(admin, boss)，方法名：delAppointmentDateById")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delBookingDateById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        // 删除这个预约日期关联的所有时间点
        QueryWrapper<BookingTime> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("bookingDateId", id);
        boolean remove = bookingTimeService.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "预约时间点删除失败");

        // 删除这个预约日期
        boolean success = bookingDateService.removeById(id);
        ThrowUtils.throwIf(!success, ErrorCode.OPERATION_ERROR, "预约日期删除失败");

        return ResultUtils.success(true);
    }




    /**
     * Web端管理员根据id删除预约时间
     * @param commonRequest 预约时间id
     * @return 是否删除成功
     */
    @PostMapping("/del/time/id")
    @Operation(summary = "Web端管理员根据id删除预约时间", description = "参数：预约时间id，权限：管理员(admin, boss)，方法名：delBookingTimeById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delBookingTimeById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        boolean remove = bookingTimeService.removeById(id);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "预约时间删除失败");

        return ResultUtils.success(true);
    }



    /**
     * Web端管理员根据id修改预约日期的状态
     * @param commonRequest 预约日期id
     * @return 是否更新成功
     */
    @PostMapping("/update/status")
    @Operation(summary = "Web端管理员根据id修改预约日期的状态", description = "参数：预约日期id，权限：管理员(admin, boss)，方法名：updateBookingTimeStatusById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateBookingTimeStatusById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        BookingDate bookingDate = bookingDateService.getById(id);
        ThrowUtils.throwIf(bookingDate == null, ErrorCode.OPERATION_ERROR, "预约日期不存在");
        Integer status = bookingDate.getIsAvailable() == 0 ? 1 : 0;
        UpdateWrapper<BookingDate> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isAvailable", status);
        boolean update = bookingDateService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "预约日期状态修改失败");

        return ResultUtils.success(true);
    }





    /**
     * Web端管理员添加预约时间
     * @param bookingTimeSingleAddRequest 预约时间段添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add/time")
    @Operation(summary = "Web端管理员添加预约时间段", description = "参数：预约时间段添加请求体，权限：管理员(admin, boss)，方法名：addAppointmentDate")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Long> addTimePeriod(@RequestBody BookingTimeSingleAddRequest bookingTimeSingleAddRequest) {
        if (bookingTimeSingleAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        BookingTime bookingTime = new BookingTime();
        BeanUtils.copyProperties(bookingTimeSingleAddRequest, bookingTime);
        boolean save = bookingTimeService.save(bookingTime);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "预约时间段添加失败");

        return ResultUtils.success(bookingTime.getId());
    }





    /**
     * Web端管理员添加预约日期
     * @param bookingDateSingleAddRequest 预约日期添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加预约日期", description = "参数：预约日期添加请求体，权限：管理员(admin, boss)，方法名：addBookingDate")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addBookingDate(@RequestBody BookingDateSingleAddRequest bookingDateSingleAddRequest) {
        if (bookingDateSingleAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 添加当前商品的预约日期
        BookingDate bookingDate = new BookingDate();
        BeanUtils.copyProperties(bookingDateSingleAddRequest, bookingDate);
        boolean save = bookingDateService.save(bookingDate);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "预约日期添加失败");

        // 添加当前预约日期的预约时间段
        Long bookingDateId = bookingDate.getId();
        List<BookingTimeAddRequest> bookingTimeAddRequestList = bookingDateSingleAddRequest.getBookingTimeAddRequestList();
        List<BookingTime> bookingTimes = bookingTimeAddRequestList.stream().map(bookingTimeAddRequest -> {
            BookingTime bookingTime = new BookingTime();
            BeanUtils.copyProperties(bookingTimeAddRequest, bookingTime);
            bookingTime.setBookingDateId(bookingDateId);
            return bookingTime;
        }).toList();
        boolean result = bookingTimeService.saveBatch(bookingTimes);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "批量添加预约时间失败");

        return ResultUtils.success(true);
    }




    /**
     * Web端管理员批量添加预约日期
     * @param bookingDateSingleAddRequestList 预约日期添加请求体列表
     * @return 是否添加成功
     */
    @PostMapping("/addBatch")
    @Operation(summary = "Web端管理员批量添加预约日期", description = "参数：预约日期添加请求体列表，权限：管理员(admin, boss)，方法名：addBatchBookingDate")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addBatchBookingDate(@RequestBody List<BookingDateSingleAddRequest> bookingDateSingleAddRequestList) {
        if (bookingDateSingleAddRequestList == null || bookingDateSingleAddRequestList.isEmpty()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 批量添加当前商品的预约日期
        List<BookingDate> bookingDateList = commonService.convertList(bookingDateSingleAddRequestList, BookingDate.class);
        boolean saveBatch = bookingDateService.saveBatch(bookingDateList);
        ThrowUtils.throwIf(!saveBatch, ErrorCode.OPERATION_ERROR, "预约日期批量添加失败");

        List<BookingTime> bookingTimes = new ArrayList<>();
        for (int i = 0; i < bookingDateSingleAddRequestList.size(); i++) {
            Long bookingDateId = bookingDateList.get(i).getId();
            List<BookingTimeAddRequest> bookingTimeAddRequestList = bookingDateSingleAddRequestList.get(i).getBookingTimeAddRequestList();
            List<BookingTime> bookingTimeList = bookingTimeAddRequestList.stream().map(bookingTimeAddRequest -> {
                BookingTime bookingTime = new BookingTime();
                BeanUtils.copyProperties(bookingTimeAddRequest, bookingTime);
                bookingTime.setBookingDateId(bookingDateId);
                return bookingTime;
            }).toList();
            bookingTimes.addAll(bookingTimeList);
        }

        boolean result = bookingTimeService.saveBatch(bookingTimes);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "预约时间批量添加失败");

        return ResultUtils.success(true);
    }







}
