package com.cultural.heritage.controller.good;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.appointment.single.AppointmentDateSingleAddRequest;
import com.cultural.heritage.model.dto.timePeriod.TimePeriodAddRequest;
import com.cultural.heritage.model.dto.timePeriod.single.TimePeriodSingleAddRequest;
import com.cultural.heritage.model.dto.timePeriod.single.TimePeriodSingleUpdateRequest;
import com.cultural.heritage.model.entity.AppointmentDate;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.PendingServiceGood;
import com.cultural.heritage.model.entity.TimePeriod;
import com.cultural.heritage.model.vo.appointment.AppointmentDateTimePeriodVO;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.good.AppointmentDateService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.good.TimePeriodService;
import com.cultural.heritage.service.order.PendingServiceGoodService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/appointmentDate")
@Slf4j
@Tag(name = "服务类商品预约日期模块")
public class AppointmentDateController {


    @Resource
    private AppointmentDateService appointmentDateService;



    @Resource
    private TimePeriodService timePeriodService;



    @Resource
    private PendingServiceGoodService pendingServiceGoodService;



    @Resource
    private GoodService goodService;



    @Resource
    private CommonService commonService;



    /**
     * Web端管理员根据id删除预约日期
     * @param commonRequest 预约日期id
     * @return 是否删除成功
     */
    @PostMapping("/del/id")
    @Operation(summary = "Web端管理员根据id删除预约日期", description = "参数：预约日期id，权限：管理员(admin, boss)，方法名：delAppointmentDateById")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delAppointmentDateById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        // 删除这个预约日期关联的所有时间段
        QueryWrapper<TimePeriod> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("appointmentDateId", id);
        boolean remove = timePeriodService.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "预约时间段删除失败");

        // 删除这个预约日期
        boolean success = appointmentDateService.removeById(id);
        ThrowUtils.throwIf(!success, ErrorCode.OPERATION_ERROR, "预约日期删除失败");

        // 删除服务类商品待处理记录
        QueryWrapper<PendingServiceGood> goodQueryWrapper = new QueryWrapper<>();
        goodQueryWrapper.eq("appointmentDateId", id);
        boolean result = pendingServiceGoodService.remove(goodQueryWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品待处理记录删除失败");

        return ResultUtils.success(true);


    }




    /**
     * Web端管理员根据id删除预约时间段
     * @param commonRequest 预约时间段id
     * @return 是否删除成功
     */
    @PostMapping("/del/time/id")
    @Operation(summary = "Web端管理员根据id删除预约时间段", description = "参数：预约日期段id，权限：管理员(admin, boss)，方法名：delTimePeriodById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delTimePeriodById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        boolean remove = timePeriodService.removeById(id);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "预约时间段删除失败");

        // 删除服务类商品待处理记录
        QueryWrapper<PendingServiceGood> goodQueryWrapper = new QueryWrapper<>();
        goodQueryWrapper.eq("timePeriodId", id);
        boolean result = pendingServiceGoodService.remove(goodQueryWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品待处理记录删除失败");

        return ResultUtils.success(true);
    }



    /**
     * Web端管理员根据id修改预约日期的状态
     * @param commonRequest 预约日期id
     * @return 是否更新成功
     */
    @PostMapping("/update/status")
    @Operation(summary = "Web端管理员根据id修改预约日期的状态", description = "参数：预约日期id，权限：管理员(admin, boss)，方法名：updateTimePeriodStatusById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateTimePeriodStatusById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        AppointmentDate appointmentDate = appointmentDateService.getById(id);
        ThrowUtils.throwIf(appointmentDate == null, ErrorCode.OPERATION_ERROR, "预约日期不存在");
        Integer status = appointmentDate.getIsAvailable() == 0 ? 1 : 0;
        UpdateWrapper<AppointmentDate> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isAvailable", status);
        boolean update = appointmentDateService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "预约日期状态修改失败");

        // 批量更新服务类商品待处理记录
        UpdateWrapper<PendingServiceGood> goodUpdateWrapper = new UpdateWrapper<>();
        goodUpdateWrapper.eq("appointmentDateId", id).set("isAvailable", status);
        boolean result = pendingServiceGoodService.update(goodUpdateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品待处理记录批量更新失败");
        return ResultUtils.success(true);
    }




    /**
     * Web端管理员根据id修改预约时间段的人数
     * @param timePeriodSingleUpdateRequest 预约日期id
     * @return 是否更新成功
     */
    @PostMapping("/update/time")
    @Operation(summary = "Web端管理员根据id修改预约时间段的人数", description = "参数：预约日期id，权限：管理员(admin, boss)，方法名：updateTimePeriodPersonNumberById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateTimePeriodPersonNumberById(@RequestBody TimePeriodSingleUpdateRequest timePeriodSingleUpdateRequest) {
        if (timePeriodSingleUpdateRequest == null || timePeriodSingleUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 根据id获取当前的预约日期
        Long id = timePeriodSingleUpdateRequest.getId();
        TimePeriod time = timePeriodService.getById(id);
        // 封装更新后的预约时间段
        TimePeriod timePeriod = new TimePeriod();
        BeanUtils.copyProperties(timePeriodSingleUpdateRequest, timePeriod);
        timePeriod.setId(time.getId());
        timePeriod.setTimeSlot(time.getTimeSlot());

        boolean update = timePeriodService.updateById(timePeriod);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "预约时间段人数修改失败");

        // 更新服务类商品待处理记录
        UpdateWrapper<PendingServiceGood> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("timePeriodId", id);
        updateWrapper.set("minNumber", timePeriodSingleUpdateRequest.getMinNumber());
        updateWrapper.set("maxNumber", timePeriodSingleUpdateRequest.getMaxNumber());
        boolean result = pendingServiceGoodService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品待处理记录更新失败");

        return ResultUtils.success(true);
    }


    /**
     * Web端管理员添加预约时间段
     * @param timePeriodSingleAddRequest 预约时间段添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add/time")
    @Operation(summary = "Web端管理员添加预约时间段", description = "参数：预约时间段添加请求体，权限：管理员(admin, boss)，方法名：addAppointmentDate")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Long> addTimePeriod(@RequestBody TimePeriodSingleAddRequest timePeriodSingleAddRequest) {
        if (timePeriodSingleAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        TimePeriod timePeriod = new TimePeriod();
        BeanUtils.copyProperties(timePeriodSingleAddRequest, timePeriod);
        boolean save = timePeriodService.save(timePeriod);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "预约时间段添加失败");

        // 添加服务类商品待处理记录
        PendingServiceGood pendingServiceGood = new PendingServiceGood();
        BeanUtils.copyProperties(timePeriodSingleAddRequest, pendingServiceGood);

        Long appointmentDateId = timePeriodSingleAddRequest.getAppointmentDateId();
        AppointmentDate appointmentDate = appointmentDateService.getById(appointmentDateId);
        ThrowUtils.throwIf(appointmentDate == null, ErrorCode.OPERATION_ERROR, "当前预约日期不存在");

        Long goodId = appointmentDate.getGoodId();
        Good good = goodService.getById(goodId);
        ThrowUtils.throwIf(good == null, ErrorCode.OPERATION_ERROR, "当前服务类商品不存在");

        pendingServiceGood.setGoodId(goodId);
        pendingServiceGood.setGoodImg(good.getGoodImg());
        pendingServiceGood.setPrice(good.getPrice());
        pendingServiceGood.setTimePeriodId(timePeriod.getId());
        pendingServiceGood.setReservationDate(appointmentDate.getSpecificDate());
        pendingServiceGood.setIsAvailable(appointmentDate.getIsAvailable());

        boolean result = pendingServiceGoodService.save(pendingServiceGood);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品待处理记录添加失败");

        return ResultUtils.success(timePeriod.getId());
    }



    /**
     * Web端管理员添加预约日期
     * @param appointmentDateSingleAddRequest 预约日期添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加预约日期", description = "参数：预约日期添加请求体，权限：管理员(admin, boss)，方法名：addAppointmentDate")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addAppointmentDate(@RequestBody AppointmentDateSingleAddRequest appointmentDateSingleAddRequest) {
        if (appointmentDateSingleAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 添加当前商品的预约日期
        AppointmentDate appointmentDate = new AppointmentDate();
        BeanUtils.copyProperties(appointmentDateSingleAddRequest, appointmentDate);
        boolean save = appointmentDateService.save(appointmentDate);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "预约日期添加失败");

        // 添加当前预约日期的预约时间段
        Long appointmentDateId = appointmentDate.getId();
        List<TimePeriodAddRequest> timePeriodAddRequestList = appointmentDateSingleAddRequest.getTimePeriodAddRequestList();
        List<TimePeriod> timePeriodList = timePeriodAddRequestList.stream().map(timePeriodAddRequest -> {
            TimePeriod timePeriod = new TimePeriod();
            BeanUtils.copyProperties(timePeriodAddRequest, timePeriod);
            timePeriod.setAppointmentDateId(appointmentDateId);
            return timePeriod;
        }).toList();
        boolean result = timePeriodService.saveBatch(timePeriodList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "批量添加预约时间段失败");

        // 批量添加服务类商品待处理记录
        Long goodId = appointmentDate.getGoodId();
        Good good = goodService.getById(goodId);
        ThrowUtils.throwIf(good == null, ErrorCode.OPERATION_ERROR, "当前商品不存在");

        // 获取预约日期列表
        List<AppointmentDate> appointmentDateList = commonService.findByFieldInTargetField(timePeriodList, appointmentDateService, TimePeriod::getAppointmentDateId, "id");

        // 封装map集合（键：预约日期id, 值：是否可预约）
        Map<Long, Integer> map = new HashMap<>();
        for (AppointmentDate appDate : appointmentDateList) {
            map.put(appDate.getId(), appDate.getIsAvailable());
        }

        List<PendingServiceGood> pendingServiceGoodList = new ArrayList<>();
        for (TimePeriod timePeriod : timePeriodList) {
            PendingServiceGood pendingServiceGood = new PendingServiceGood();
            pendingServiceGood.setGoodId(goodId);
            pendingServiceGood.setGoodImg(good.getGoodImg());
            pendingServiceGood.setPrice(good.getPrice());
            pendingServiceGood.setAppointmentDateId(appointmentDate.getId());
            pendingServiceGood.setReservationDate(appointmentDate.getSpecificDate());
            pendingServiceGood.setIsAvailable(map.get(appointmentDate.getId()));

            pendingServiceGood.setTimePeriodId(timePeriod.getId());
            pendingServiceGood.setTimeSlot(timePeriod.getTimeSlot());
            pendingServiceGood.setMinNumber(timePeriod.getMinNumber());
            pendingServiceGood.setMaxNumber(timePeriod.getMaxNumber());

            pendingServiceGoodList.add(pendingServiceGood);
        }
        boolean saveBatch = pendingServiceGoodService.saveBatch(pendingServiceGoodList);
        ThrowUtils.throwIf(!saveBatch, ErrorCode.OPERATION_ERROR, "服务类商品待处理记录批量添加失败");

        return ResultUtils.success(true);
    }




    /**
     * Web端管理员批量添加预约日期
     * @param appointmentDateSingleAddRequestList 预约日期添加请求体列表
     * @return 是否添加成功
     */
    @PostMapping("/addBatch")
    @Operation(summary = "Web端管理员批量添加预约日期", description = "参数：预约日期添加请求体列表，权限：管理员(admin, boss)，方法名：addBatchAppointmentDate")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addBatchAppointmentDate(@RequestBody List<AppointmentDateSingleAddRequest> appointmentDateSingleAddRequestList) {
        if (appointmentDateSingleAddRequestList == null || appointmentDateSingleAddRequestList.isEmpty()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long goodId = appointmentDateSingleAddRequestList.get(0).getGoodId();
        Good good = goodService.getById(goodId);
        ThrowUtils.throwIf(good == null, ErrorCode.OPERATION_ERROR, "当前商品不存在");

        // 批量添加当前商品的预约日期
        List<AppointmentDate> appointmentDateList = commonService.convertList(appointmentDateSingleAddRequestList, AppointmentDate.class);
        boolean save = appointmentDateService.saveBatch(appointmentDateList);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "预约日期批量添加失败");

        // 批量添加当前预约日期的预约时间段
        List<TimePeriod> timePeriods = new ArrayList<>();
        for (int i = 0; i < appointmentDateList.size(); i++) {
            Long appointmentDateId = appointmentDateList.get(i).getId();
            List<TimePeriodAddRequest> timePeriodAddRequestList = appointmentDateSingleAddRequestList.get(i).getTimePeriodAddRequestList();
            List<TimePeriod> timePeriodList = timePeriodAddRequestList.stream().map(timePeriodAddRequest -> {
                TimePeriod timePeriod = new TimePeriod();
                BeanUtils.copyProperties(timePeriodAddRequest, timePeriod);
                timePeriod.setAppointmentDateId(appointmentDateId);
                return timePeriod;
            }).toList();
            timePeriods.addAll(timePeriodList);
        }
        boolean result = timePeriodService.saveBatch(timePeriods);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "批量添加预约时间段失败");

        // 批量插入服务类商品待处理记录
        List<AppointmentDateTimePeriodVO> appointmentDateTimePeriodVOList = appointmentDateService.queryAppointmentDateDetailById(good.getId());
        List<PendingServiceGood> pendingServiceGoodList = new ArrayList<>();
        for (AppointmentDateTimePeriodVO appointmentDateTimePeriodVO : appointmentDateTimePeriodVOList) {
            PendingServiceGood pendingServiceGood = new PendingServiceGood();
            BeanUtils.copyProperties(appointmentDateTimePeriodVO, pendingServiceGood);
            pendingServiceGood.setId(null);
            pendingServiceGood.setGoodImg(good.getGoodImg());
            pendingServiceGood.setPrice(good.getPrice());
            pendingServiceGood.setReservationDate(appointmentDateTimePeriodVO.getSpecificDate());
            pendingServiceGood.setAppointmentDateId(appointmentDateTimePeriodVO.getId());
            pendingServiceGoodList.add(pendingServiceGood);
        }
        boolean batch = pendingServiceGoodService.saveBatch(pendingServiceGoodList);
        ThrowUtils.throwIf(!batch, ErrorCode.OPERATION_ERROR, "服务类商品待处理记录批量插入失败");


        return ResultUtils.success(true);
    }

}
