package com.cultural.heritage.controller.good;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.cart.CartRecordAddRequest;
import com.cultural.heritage.model.dto.cart.CartRecordUpdateRequest;
import com.cultural.heritage.model.dto.order.BuySingleGoodVerifyRequest;
import com.cultural.heritage.model.entity.CartRecord;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.cart.CartGoodVO;
import com.cultural.heritage.model.vo.cart.CartOrderVO;
import com.cultural.heritage.model.vo.cart.CartRecordVO;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.*;

@RestController
@RequestMapping("/cart")
@Slf4j
@Tag(name = "常规类购物车模块")
public class CartRecordController {


    @Resource
    private CommonService commonService;


    @Resource
    private CartRecordService cartRecordService;


    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;

    /**
     * 小程序端用户添加常规类商品至购物车
     * @param cartRecordAddRequest  常规类购物车商品添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "小程序端用户添加常规类商品至购物车", description = "参数：常规类购物车商品添加请求体，权限：所有人，方法名：addCartRecord")
    public BaseResponse<Boolean> addCartRecord(@RequestBody CartRecordAddRequest cartRecordAddRequest, HttpServletRequest request) {
        if (cartRecordAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 更新购物车的商品属性
        cartRecordService.updateCartGoodAttribute(cartRecordAddRequest, request);
        return ResultUtils.success(true);
    }


    /**
     * 小程序端用户更新常规类购物车中的商品数量
     * @param cartRecordUpdateRequestList  常规类购物车更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "小程序端用户更新常规类购物车中的商品数量", description = "参数：常规类购物车更新请求体，权限：所有人，方法名：updateCartRecord")
    public BaseResponse<Boolean> updateCartRecord(@RequestBody List<CartRecordUpdateRequest> cartRecordUpdateRequestList, HttpServletRequest request) {
        if (CollectionUtils.isEmpty(cartRecordUpdateRequestList)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);

        // 获取购物车列表
        List<CartRecord> cartRecordList = commonService.findByFieldInTargetField(cartRecordUpdateRequestList, cartRecordService, CartRecordUpdateRequest::getId, "id");

        // 封装map集合（键：购物车id, 值：商品数量）
        Map<Long, Integer> QuantityMap = new HashMap<>();
        for (CartRecordUpdateRequest cartRecordUpdateRequest : cartRecordUpdateRequestList) {
            QuantityMap.put(cartRecordUpdateRequest.getId(), cartRecordUpdateRequest.getQuantity());
        }

        // 获取商品列表
        List<Good> goodList = commonService.findByFieldInTargetField(cartRecordList, goodService, CartRecord::getGoodId, "id");

        // 封装map集合（键：商品id, 值：商品价格）
        Map<Long, BigDecimal> priceMap = new HashMap<>();
        for (Good good : goodList) {
            priceMap.put(good.getId(), good.getPrice());
        }

        // 批量更新购物车数量和小计
        for (CartRecord cartRecord : cartRecordList) {
            Long cartRecordId = cartRecord.getId();
            Integer quantity = QuantityMap.get(cartRecordId);
            cartRecord.setQuantity(quantity);

            Long goodId = cartRecord.getGoodId();
            BigDecimal price = priceMap.get(goodId);
            cartRecord.setSubtotal(price.multiply(BigDecimal.valueOf(quantity)));
        }

        boolean result = cartRecordService.updateBatchById(cartRecordList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 小程序端用户删除常规类购物车中的物品
     * @param commonBatchRequest  删除的商品id列表
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "小程序端用户删除常规类购物车中的物品", description = "参数：购物车记录删除请求体(商品id列表，如[2,3,4])，权限：所有人，方法名：deleteCart")
    public BaseResponse<Boolean> deleteCart(@RequestBody CommonBatchRequest commonBatchRequest, HttpServletRequest request) {
        if (commonBatchRequest == null || CollectionUtils.isEmpty(commonBatchRequest.getIdList())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        List<Long> idList = commonBatchRequest.getIdList();
        boolean result = cartRecordService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 小程序端用户查询常规类购物车中的商品
     * @return 商品列表信息
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端用户查询常规类购物车中的商品", description = "参数：无，权限：所有人，方法名：listUserCartRecord")
    public BaseResponse<List<CartRecordVO>> listUserCartRecord(HttpServletRequest request) {
        // 根据用户id查询购物车记录
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.orderByDesc("id");
        List<CartRecord> list = cartRecordService.list(queryWrapper);
        // 封装用户购物车中的商品信息
        List<CartRecordVO> cartRecordVOList = cartRecordService.transformToCartRecordVOList(list);
        return ResultUtils.success(cartRecordVOList);
    }




    /**
     * 小程序端显示常规类购物车中不合理的商品项
     * @return  存在问题的购物车id列表
     */
    @PostMapping("/submit/error")
    @Operation(summary = "小程序端显示常规类购物车中不合理的商品项", description = "参数:无，权限：所有人，方法名：listInvalidCartIds")
    public BaseResponse<List<Long>> listInvalidCartIds (HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        List<CartRecord> cartRecords = commonService.findByFieldEqTargetField("userId", userId, cartRecordService);
        // 找出存在问题的购物车id列表
        List<Long> invalidCartIds = cartRecordService.getInvalidCartIds(cartRecords);
        return ResultUtils.success(invalidCartIds);
    }





    /**
     * 小程序端显示常规类购物车提交订单页的商品信息
     * @param commonBatchRequest    购物车id列表
     * @return  订单商品信息列表
     */
    @PostMapping("/submit/list")
    @Operation(summary = "小程序端显示常规类购物车提交订单页的商品信息", description = "参数:购物车id列表，权限：所有人，方法名：listCartRecord")
    public BaseResponse<List<CartOrderVO>> listCartRecord(@RequestBody CommonBatchRequest commonBatchRequest, HttpServletRequest request) {
        if (commonBatchRequest == null || CollectionUtils.isEmpty(commonBatchRequest.getIdList())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        List<Long> cartRecordIds = commonBatchRequest.getIdList();
        // 根据id列表获取购物车信息列表
        List<CartRecord> cartRecords = commonService.findByFieldInTargetField(cartRecordIds, cartRecordService, id -> id, "id");

        // 提取出购物车中的商品列表
        List<Good> goodList = commonService.findByFieldInTargetField(cartRecords, goodService, CartRecord::getGoodId, "id");

        // 封装goodMap(根据商品id查询商品)
        Map<Long, Good> goodMap = new HashMap<>();
        for (Good good : goodList) {
            goodMap.put(good.getId(), good);
        }
        // 校验购物车商品
        for (CartRecord cartRecord : cartRecords) {
            Good good = goodMap.get(cartRecord.getGoodId());
            Integer quantity = cartRecord.getQuantity();
            goodService.checkGoodIsExist(good);
            goodService.checkInventoryIsEnough(quantity, good.getInventory());
        }

        // 封装提交订单页的商品信息列表
        List<CartOrderVO> cartOrderVOList = new ArrayList<>();
        for (CartRecord cartRecord : cartRecords) {
            Integer quantity = cartRecord.getQuantity();
            Long goodId = cartRecord.getGoodId();
            Good good = goodMap.get(goodId);

            CartGoodVO cartGoodVO = new CartGoodVO();
            BeanUtils.copyProperties(good, cartGoodVO);

            CartOrderVO cartOrderVO = new CartOrderVO();
            cartOrderVO.setCartGoodVO(cartGoodVO);
            cartOrderVO.setQuantity(quantity);
            cartOrderVOList.add(cartOrderVO);
        }
        Collections.reverse(cartOrderVOList);
        return ResultUtils.success(cartOrderVOList);
    }





    /**
     * 小程序端用户点击立即购买触发的校验
     * @param buySingleGoodVerifyRequest    商品id和购买数量
     * @return  是否通过校验
     */
    @PostMapping("/verify/good")
    @Operation(summary = "小程序端用户点击立即购买触发的校验", description = "参数:商品id和购买数量，权限：所有人，方法名：verifyPurchaseInfo")
    public BaseResponse<Boolean> verifyPurchaseInfo(@RequestBody BuySingleGoodVerifyRequest buySingleGoodVerifyRequest, HttpServletRequest request) {
        if (buySingleGoodVerifyRequest == null || buySingleGoodVerifyRequest.getGoodId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long goodId = buySingleGoodVerifyRequest.getGoodId();
        Integer quantity = buySingleGoodVerifyRequest.getQuantity();
        Good good = goodService.getById(goodId);
        goodService.checkGoodIsExist(good);
        goodService.checkInventoryIsEnough(quantity, good.getInventory());
        return ResultUtils.success(true);
    }

}
