package com.cultural.heritage.controller.order;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.order.BookingOrderQueryRequest;
import com.cultural.heritage.model.entity.PendingServiceGood;
import com.cultural.heritage.model.entity.PendingServiceOrder;
import com.cultural.heritage.model.vo.order.PendingServiceGoodVO;
import com.cultural.heritage.model.vo.order.PendingServiceOrderVO;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.order.PendingServiceGoodService;
import com.cultural.heritage.service.order.PendingServiceOrderService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/pending")
@Slf4j
@Tag(name = "待处理商品管理模块")
@Transactional(rollbackFor = Exception.class)
public class PendingServiceGoodController {


    @Resource
    private PendingServiceGoodService pendingServiceGoodService;


    @Resource
    private PendingServiceOrderService pendingServiceOrderService;


    @Resource
    private CommonService commonService;


    @Resource
    private UserService userService;


    /**
     * Web端管理员查询预约情况
     * @param bookingOrderQueryRequest  服务类商品预约情况查询请求体
     * @return 服务类商品预约情况列表
     */
    @PostMapping("/list/advance/detail")
    @Operation(summary = "Web端管理员查询预约情况", description = "参数：服务类商品预约情况查询请求体，权限：管理员(admin, boss)，方法名：listOrder")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<PendingServiceGoodVO>> listBookingOrderDetail(@RequestBody BookingOrderQueryRequest bookingOrderQueryRequest) {
        if (bookingOrderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        QueryWrapper<PendingServiceGood> queryWrapper = pendingServiceGoodService.getQueryWrapper(bookingOrderQueryRequest);
        long current = bookingOrderQueryRequest.getCurrent();
        long pageSize = bookingOrderQueryRequest.getPageSize();
        Page<PendingServiceGood> page = pendingServiceGoodService.page(new Page<>(current, pageSize), queryWrapper);

        List<PendingServiceGood> pendingServiceGoodList = page.getRecords();
        // 获取服务类待处理订单
        List<PendingServiceOrder> pendingServiceOrderList = commonService.findByFieldInTargetField(pendingServiceGoodList, pendingServiceOrderService, PendingServiceGood::getId, "pendingId");
        // 封装成服务类待处理订单VO
        List<PendingServiceOrderVO> pendingServiceOrderVOS = commonService.convertList(pendingServiceOrderList, PendingServiceOrderVO.class);
        // 封装map集合（键：服务类商品待处理id，值：待处理服务类订单列表）
        Map<Long, List<PendingServiceOrderVO>> map = new HashMap<>();
        for (PendingServiceOrderVO pendingServiceOrderVO : pendingServiceOrderVOS) {
            Long pendingId = pendingServiceOrderVO.getPendingId();
            List<PendingServiceOrderVO> pendingServiceOrderVOList = map.get(pendingId);
            if (pendingServiceOrderVOList == null) {
                pendingServiceOrderVOList = new ArrayList<>();
            }
            pendingServiceOrderVOList.add(pendingServiceOrderVO);
            map.put(pendingId, pendingServiceOrderVOList);
        }

        List<PendingServiceGoodVO> pendingServiceGoodVOList = new ArrayList<>();
        for (PendingServiceGood pendingServiceGood : pendingServiceGoodList) {
            PendingServiceGoodVO pendingServiceGoodVO = new PendingServiceGoodVO();
            BeanUtils.copyProperties(pendingServiceGood, pendingServiceGoodVO);
            Long goodId = pendingServiceGood.getId();
            List<PendingServiceOrderVO> pendingServiceOrderVOList = map.get(goodId);
            if (pendingServiceOrderVOList == null) continue;
            pendingServiceGoodVO.setPendingServiceOrderVOList(pendingServiceOrderVOList);
            pendingServiceGoodVOList.add(pendingServiceGoodVO);
        }

        Page<PendingServiceGoodVO> voPage = new Page<>();
        voPage.setRecords(pendingServiceGoodVOList);
        voPage.setTotal(page.getTotal());
        voPage.setSize(page.getSize());
        voPage.setCurrent(page.getCurrent());
        voPage.setPages(page.getPages());
        return ResultUtils.success(voPage);
    }


    /**
     * 小程序端用户查看当前商品的预约情况
     * @param commonRequest  商品id
     * @return 当前商品不同时间段的预约人数（键：预约日期&预约时间段，值：剩余可预约人数）
     */
    @PostMapping("/list/advance/count")
    @Operation(summary = "小程序端用户查看当前预约人数", description = "参数：服务类商品预约情况查询请求体，权限：管理员(admin, boss)，方法名：listOrder")
    public BaseResponse<Map<String, Integer>> getCurrentBookingNumberMap(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = commonRequest.getId();
        // 获取当前服务类商品的预约时间段
        List<PendingServiceGood> pendingServiceGoodList = commonService.findByFieldEqTargetField("goodId", id, pendingServiceGoodService);
        // 获取当前服务类商品的预约情况
        List<PendingServiceOrder> pendingServiceOrderList = commonService.findByFieldInTargetField(pendingServiceGoodList, pendingServiceOrderService, PendingServiceGood::getId, "pendingId");
        // 封装Map集合（键：服务类商品待处理id，值：预约人数）
        Map<Long, Integer> countMap = new HashMap<>();
        for (PendingServiceOrder pendingServiceOrder : pendingServiceOrderList) {
            Long pendingId = pendingServiceOrder.getPendingId();
            Integer quantity = pendingServiceOrder.getQuantity();
            Integer currentNumber = countMap.get(pendingId);
            countMap.put(pendingId, currentNumber == null ? quantity : currentNumber + quantity);
        }
        // 封装Map集合（键：预约日期&预约时间段，值：剩余可预约人数）
        Map<String, Integer> bookingNumberMap = new HashMap<>();
        for (PendingServiceGood pendingServiceGood : pendingServiceGoodList) {
            Long pendingId = pendingServiceGood.getId();
            Integer count = countMap.get(pendingId);
            String reservationDate = pendingServiceGood.getReservationDate();
            String timeSlot = pendingServiceGood.getTimeSlot();
            Integer maxNumber = pendingServiceGood.getMaxNumber();
            bookingNumberMap.put(reservationDate + "&" + timeSlot, count == null ? maxNumber : maxNumber - count);
        }
        return ResultUtils.success(bookingNumberMap);
    }


}
